--		Title:		Quake3 Arena Animation Helper for 3D Studio Max 2x and up.

--		Author:		Suren Pillay (Tracer), leader of Team Nightrunner

--		E-Mail: 	tracer01@netactive.co.za, pillays49@eng.und.ac.za, pillays@is.und.ac.za

--		Site:		www.quake3mods.com/nightrunner

--		Legal:		1.	This software is provided as is, with no warranty whatsoever (either expressed
--						or implied).  
--					2.	The developer takes no responsibility of any damage that may arise
--						from the use of this software.  
--					3.  This software may not be altered in any way or included in any publication or 
--						compilation of software unless written permission has been obtained from the 
--						developer.
--					4.  This software my only be distributed by electronic means, and may only be made
--						available for download from sites authorised by the author.
						
--==========================================================================================================--

clearlistener()
gc()
struct animseq (start, length, loop, fps, name);
struct seqdisplay(start, length, loop, fps, update);
global	currentofs = 0,
		currentstring,
		sex = "male",
		footsteps = "none",
		headoffset = [8, 8, 8] as point3,
		tabchar = "\t" as string
		seqnum = 1
		seqs = #()
		sequencelistdata = #()
		
		currentseq = animseq start:0
		seqdatatext = "" as string
		animfile = openfile "animation.cfg"
		filename = "" as string
		
function nextline =
	(
		local string outstring
		outstring = readline animfile
		while ((outstring[1] == "/") or (outstring == "")) do 
			(
				outstring = readline animfile
			)
		currentofs = 1
		outstring
	)

function nextword instring offset = 
	(
		local ofs
		ofs = currentofs
		while ((ofs <= instring.count) and ((instring[ofs] == " ") or (instring[ofs] == tabchar))) do
			(
				ofs = ofs + 1
			)
		c = ofs	
		while ((c <= instring.count) and (instring[c] != " ") and (instring[c] != tabchar)) do
			(
				c = c + 1
			)
		currentofs = c + 1
		substring instring ofs (c - ofs)
	)

function processword word = 
	(
		word = word as string
		if (word == "sex") then  
			(
				word = nextword currentstring currentofs -- complete freedom of sexual orientation
				if (word == "m") then sex = "m"
				else sex = "f"
			)
		else if (word == "footsteps") then
			(
				word = nextword currentstring currentofs
				footsteps = word
			)
		else if (word == "headoffset") then
			(
				
				headoffset.x = nextword currentstring currentofs as float
				headoffset.y = nextword currentstring currentofs as float
				headoffset.z = nextword currentstring currentofs as float
--				print headoffset
			)
		else
			(
				
				currentseq.start = word as float
				currentseq.length = nextword currentstring currentofs as float
				currentseq.loop = nextword currentstring currentofs as float
				currentseq.fps = nextword currentstring currentofs as float
				word = nextword currentstring currentofs -- the // is discardable
				currentseq.name = nextword currentstring currentofs as string
				seqs[seqnum] = animseq start:currentseq.start length:currentseq.length loop:currentseq.loop fps:currentseq.fps name:currentseq.name
				seqnum = seqnum + 1
			)
				
	)

-- for debugging
function writeseqs = 
	(
		for c = 1 to (seqs.count-1) do
		print((c as string) + "\t" + (seqs[c] as string))
	)

function setseq num = 
	(
		animationrange = interval  (seqs[num].start) (seqs[num].start + seqs[num].length)
	)	

-- ///////////////////

function loadfile filename = 
	(
		if (filename != undefined) then
		(
			filerror = false
			try (animfile = openfile filename) 
			catch 
				(
					filerror = true
					messagebox "There was an error loading the file.  Check that the file exists, is not write protected and is not in use" title:"Error" beep:true
				)
				
			while (not(eof animfile) and not(filerror)) do
				(
					currentstring = nextline()
					currentword = nextword currentstring currentofs
					processword currentword
				)
		) 
	)

--==========================================================================================================--

rollout aboutrollout "About"
	(
		bitmap logo_bitmap filename:(scriptspath + "Q3A Helper/logo.bmp")
	)

function createlabel type num = 
	(
		val = execute ("seqs[" + num as string + "]." + type as string + " as string")
		data = "label data_" + type + num as string + " \"" + val + "\" across:5 width:50 "
		if (type == "start") then x = 20
		else if (type == "length") then x = 70
		else if (type == "loop") then x = 120
		else if (type == "fps") then x = 170

		data = data + "pos:[" + x as string + ",10 + 20*" + num as string + "]"
		data = data + "\n"
		data
	)
	
function createbutton num =
	(
		val = execute ("seqs[" + num as string + "].name as string")
		data = "button data_" + num as string + " \"" + val + "\" across:5 height:18 width:120 "
		data = data + "pos:[220,10 + 20*" + num as string + "]"
		data = data + "\n"
		data = data + "on data_" + num as string + " pressed do setseq " + num as string + "\n"
		data
	)

function createseqdata = 
	(
		rollstring = "rollout seqrollout \"Sequences\" (\n"

		rollstring = rollstring + "label head_start \"Start\" pos:[20, 10] width:50 height:20\n"
		rollstring = rollstring + "label head_length \"Length\" pos:[70, 10] width:50 height:20\n"
		rollstring = rollstring + "label head_loop \"Loop\" pos:[120, 10] width:50 height:20\n"
		rollstring = rollstring + "label head_fps \"FPS\" pos:[170, 10] width:50 height:20\n"
		rollstring = rollstring + "label head_name \"Name\" pos:[220, 10] width:120 height:20\n"


		for c = 1 to (seqs.count-1) do
			(
				rollstring = rollstring + createlabel "start" c + createlabel "length" c + createlabel "loop" c + createlabel "fps" c + createbutton c
				
			)
		rollstring = rollstring + ")"
		rollstring
	)	


rollout filerollout "Files"
	(
		button loadcfg "Load Config" across:2 width:100 height:20
	 	button savecfg "Save Config" width:100 height:20 
		on savecfg pressed do
			(
				filename = getSaveFileName types:"Q3A Animation Config(*.cfg)|*.cfg|All files(*.*)|*.*|"
				if (filename == undefined) then
					(
						messagebox "File creation error.  User cancelled operation." title:"Error" beep:true
					)
				else
					(
						filerror = false
						try (animfile = createfile filename)
						catch 
							(
								filerror = true
								messagebox "File creation error.  Check that the file is not write protected or in use." title:"Error" beep:true
							)
						if (not(filerror)) then
							(
								format "// animation config file\n\n" to:animfile
								format "sex %\n" sex to:animfile
								format "headoffset % % %\n" (headoffset.x as integer) (headoffset.y as integer) (headoffset.z as integer) to:animfile
								format "footsteps %\n\n" footsteps to:animfile
								format "// first frame, num frames, looping frames, frames per second\n\n" to:animfile
								for c = 1 to (seqs.count-1) do
									(
										format "%\t" (seqs[c].start as integer) to:animfile
										format "%\t" (seqs[c].length as integer) to:animfile
										format "%\t" (seqs[c].loop as integer) to:animfile
										format "%\t\t" (seqs[c].fps as integer) to:animfile
										format "// %\n" (seqs[c].name as string) to:animfile
									)
								close animfile
							)
					)
			)
		on loadcfg pressed do
			(	
				filename = getOpenFileName types:"Q3A Animation Config(*.cfg)|*.cfg|All files(*.*)|*.*|"
				if (filename != #undefined) then 
					(
						seqnum = 1
						loadfile filename
						removerollout seqrollout mainfloater
						execute(createseqdata())
						addrollout seqrollout mainfloater
					)
				else
					(
						messagebox "File load error.  User cancelled operation." title:"Error" beep:true
					)
			) 
	)


function assigneditsequencelist =
	(
		for c = 1 to seqs.count-1 do 
			append sequencelistdata seqs[c].name
	)

rollout editseqrollout "Edit"
	(
		local sel = 1 as integer
		listbox sequencelist selection:1 height:6 items:sequencelistdata pos:[10,10] width:150 caption:"Loaded Sequences"
		spinner editstart "Start: \t" type:#integer scale:1 fieldwidth:50 offset:[-30, -85] range:[0,5000,seqs[1].start]
		spinner editlength "Length: \t" type:#integer scale:1 fieldwidth:50 offset:[-30,0] range:[0,5000,seqs[1].length]
		spinner editloop "Loop: \t" type:#integer scale:1 fieldwidth:50 offset:[-30,0] range:[0,5000,seqs[1].loop]
		spinner editfps "FPS: \t" type:#integer scale:1 fieldwidth:50 offset:[-30,0] range:[0,5000,seqs[1].fps]
		label n1 "New Frames:" across:2 offset:[-30, 10]
		radiobuttons resizemodeselect selection:1 columns:4 labels:#("Stretch", "Front", "Back", "Time") offset:[-30, 10] caption:"text"
		button editaccept "Accept" across:5 height:18 width:60 
		button editreset "Reset"  height:18 width:60
		button editviewfull "Vw Full" height:18 width:60
		button editviewloop "Vw Loop"  height:18 width:60
		button editupdate "Update"  height:18 width:60

		on sequencelist selected selectionzz do
			(
				
				sel = sequencelist.selection
--				print(sel)
				editstart.value = seqs[sel].start
				editlength.value = seqs[sel].length
				editloop.value = seqs[sel].loop
				editfps.value = seqs[sel].fps
			)
		on editaccept pressed do
			(
--				print(sel);
				if (seqs[sel].start != editstart.value) then
					(
						difference = editstart.value - seqs[sel].start	
							if (difference > 0) then
								(
									inserttime $* seqs[sel].start difference
									for c = sel to (seqs.count-1) do
										seqs[c].start = seqs[c].start + difference
									setseq sel
									execute(createseqdata())
									assigneditsequencelist()
								)
							else
								(
									deletetime $* (interval (seqs[sel].start + difference) (seqs[sel].start)) #incleft
									for c = sel to (seqs.count-1) do
										seqs[c].start = seqs[c].start + difference
									setseq sel
									execute(createseqdata())
									assigneditsequencelist()

								)
							
					)
				if (seqs[sel].length != editlength.value) then
					(
						difference = editlength.value - seqs[sel].length
						if (resizemodeselect.state == 1) then --stretch
							(
								time_scale = (editlength.value / seqs[sel].length) as float
--								print ("Time scale:  " + time_scale as string)
								I = (interval (seqs[sel].start) (seqs[sel].start + seqs[sel].length))
--								print ("Interval:  " + I as string);
								scaletime $* I time_scale
								seqs[sel].length = seqs[sel].length * time_scale								for c = (sel + 1) to (seqs.count-1) do
									seqs[c].start = seqs[c].start + difference
								setseq sel
							)
						else if (resizemodeselect.state == 2) then --front
							(
								if (difference > 0) then
									(
										inserttime $* seqs[sel].start difference
										seqs[sel].length = editlength.value									
										for c = (sel + 1) to (seqs.count-1) do
											seqs[c].start = seqs[c].start + difference
										setseq sel
									)
								else
									(
										deletetime $* (interval (seqs[sel].start) (seqs[sel].start - difference))
										seqs[sel].length = editlength.value
										for c = (sel + 1) to (seqs.count-1) do
											seqs[c].start = seqs[c].start + difference
										setseq sel
									)
							)
						else if (resizemodeselect.state == 3) then --back
							(
								if (difference > 0) then
									(
										inserttime $* (seqs[sel].start + seqs[sel].length + 1) difference
										seqs[sel].length = editlength.value									
										for c = (sel + 1) to (seqs.count-1) do
											seqs[c].start = seqs[c].start + difference
										setseq sel
									)
								else
									(
										deletetime $* (interval (seqs[sel].start + seqs[sel].length + difference) (seqs[sel].start + seqs[sel].length + 1)) #incleft
										seqs[sel].length = editlength.value
										for c = (sel + 1) to (seqs.count-1) do
											seqs[c].start = seqs[c].start + difference
										setseq sel
									)


							)
						else --time
							(
								if (difference > 0) then
									(
										inserttime $* slidertime difference
										seqs[sel].length = editlength.value									
										for c = (sel + 1) to (seqs.count-1) do
											seqs[c].start = seqs[c].start + difference
										setseq sel
									)
								else
									(
										deletetime $* (interval (slidertime) (slidertime-difference))
										seqs[sel].length = editlength.value
										for c = (sel + 1) to (seqs.count-1) do
											seqs[c].start = seqs[c].start + difference
										setseq sel
									)
							)
					)
			)
		on editupdate pressed do
			(
				removerollout seqrollout mainfloater
				execute(createseqdata())
				addrollout seqrollout mainfloater
			)
		on editviewfull pressed do
			(
				animationrange = interval seqs[sel].start (seqs[sel].start + seqs[sel].length)
			)
		on editviewloop pressed do
			(
				if (seqs[sel].loop > 0) then -- to catch errors setting active segment to zero length
					(
						animationrange = interval seqs[sel].start (seqs[sel].start + seqs[sel].loop)
					)
			)
		on editreset pressed do
			(
				editstart.value = seqs[sel].start
				editlength.value = seqs[sel].length
				editloop.value = seqs[sel].loop
				editfps.value = seqs[sel].fps
			)
		
	)
	
rollout helperrollout "Helpers"
	(
		button tag "Tag" across:3 width:100 height:18 enabled:false
	 	button treadmill "Treadmill" across:3 width:100 height:18 enabled:false
		button boundingbox "Bounding Box" across:3 width:100 height:18 enabled:false
		button weapon "Weapon" across:3 width:100 height:18 enabled:false
		button viewcam "View Cam" across:3 width:100 height:18 enabled:false
		button uvwdistorter "UVW Distort" across:3 width:100 height:18 enabled:false

	)




loadfile "animation.cfg"
execute(createseqdata())
assigneditsequencelist()
mainfloater = newrolloutfloater "Q3A Helper" 380 200
addrollout aboutrollout mainfloater
addrollout filerollout mainfloater
addrollout editseqrollout mainfloater rolledup:true
addrollout helperrollout mainfloater rolledup:true
addrollout seqrollout mainfloater


--==========================================================================================================--


